function CustomButtonObject(uri,
                            localname,
                            label,tooltiptext,accesskey,
                            cssclass,style,image,hidden,
                            oncommand,onclick,
                            ondragover,ondragdrop,ondraggesture,ondragenter,ondragexit,
                            type,disabled,observes,context)
{
  if(!this.init()) return null;
  this.properties= new Array();
  this.properties["localname"]     = (localname)?     localname.toString()    : "button";
  this.properties["label"]         = (label)?         label.toString()        : "New Item";
  this.properties["class"]         = (cssclass)?     cssclass.toString()      : "toolbarbutton-custombutton";
  this.properties["style"]         = (style)?         style.toString()        : "";
  this.properties["image"]         = (image)?         image.toString()        : "";
  this.properties["tooltiptext"]   = (tooltiptext)?  tooltiptext.toString()   : "New Item";
  this.properties["accesskey"]     = (accesskey)?     accesskey.toString()    : "";
  this.properties["oncommand"]     = (oncommand)?     oncommand.toString()    : "window.custombutton.settings()\;";
  this.properties["onclick"]       = (onclick)?       onclick.toString()      : "";
  this.properties["ondragover"]    = (ondragover)?    ondragover.toString()   : "nsDragAndDrop.dragOver(event,custombuttonDNDObserver)";
  this.properties["ondragdrop"]    = (ondragdrop)?    ondragdrop.toString()   : "nsDragAndDrop.drop(event,custombuttonDNDObserver)";
  this.properties["ondraggesture"] = (ondraggesture)? ondraggesture.toString() : "nsDragAndDrop.startDrag(event,custombuttonDNDObserver)";
  this.properties["ondragenter"]   = (ondragenter)?   ondraenter.toString()   : "";
  this.properties["ondragexit"]    = (ondragexit)?    ondragexit.toString()   : "";
  this.properties["hidden"]        = (hidden && hidden != "false")?   "true"  : "false";
  this.properties["type"]          = (type)?         type.toString()          : "";
  this.properties["disabled"]      = (disabled && disabled != "false")? "true"  : "false";
  this.properties["observes"]      = (observes)?  observes.toString()         : "";
  this.properties["context"]       = (context)?    context.toString()         : "";
  this.uri                         = (uri) ?          uri                     : "urn:custombutton:"+custombuttonObjectService.IDmaker();
}

CustomButtonObject.prototype=
{
  uri:null,
  properties:null,

  append: function()
  {
    custombuttonObjectService.appendObject(this);
  },
  remove: function()
  {
    custombuttonObjectService.removeObject(this);
  },
  read: function()
  {
    custombuttonObjectService.readObject(this);
  },

  init : function()
  {
    if(typeof window.custombuttonObjectService =="undefined") return false;
    if(!window.custombuttonObjectService.RDFService) window.custombuttonObjectService.init();
    return true;
  },
  toString:function()
  {
    return "[object CustomButtonObject]\n"+ ((this.uri)? this.uri : "undefined");
  }
};

/* kitchen sink RDF Functions */
var custombuttonObjectService={

  RDFService  :  null,
  DataSource  :  null,
  Container  :  null,
  nsIWindowMediator : null,

  init: function()
  {
    //// INIT this.Datasource////////
    this.RDFService = Components.classes["@mozilla.org/rdf/rdf-service;1"]
                      .getService(Components.interfaces.nsIRDFService);
    this.DataSource=this.RDFService.GetDataSource( "rdf:local-store" );
  
    //// INIT this.Container ///////
    this.Container = Components.classes["@mozilla.org/rdf/container;1"]
                    .getService(Components.interfaces.nsIRDFContainer);
    var RDFCUtils = Components.classes['@mozilla.org/rdf/container-utils;1']
                    .getService(Components.interfaces.nsIRDFContainerUtils);
    var ContainerResource=this.RDFService.GetResource("urn:custombutton:buttons");

    if( RDFCUtils.IsContainer(this.DataSource,ContainerResource ) )
          this.Container.Init(this.DataSource, ContainerResource);
    else  this.Container=RDFCUtils.MakeSeq(this.DataSource,ContainerResource);

    //WindowMediator for render buttons after changes.
    this.nsIWindowMediator = Components.classes['@mozilla.org/appshell/window-mediator;1']
                  .getService(Components.interfaces.nsIWindowMediator);

  },

  removeObject : function (targetObject)
  {
    var itemResource=this.RDFService.GetResource(targetObject.uri);
    if(this.Container.IndexOf(itemResource)<0) return;

    this.Container.RemoveElement(itemResource,true);
    this.Unassert(this.DataSource,itemResource);

    this.updateWindow();
  },

  appendObject : function (targetObject)
  {
    var itemResource=this.RDFService.GetResource(targetObject.uri);

    if (this.Container.IndexOf(itemResource)<0)  this.Container.AppendElement(itemResource);
    else            this.Unassert(this.DataSource,itemResource);

    var property=null;
    for(var i in targetObject.properties){
      property=this.RDFService.GetResource("http://forums.mozillazine.org/rdf/custombutton#"+i);
      this.DataSource.Assert(itemResource, property,  this.RDFService.GetLiteral(targetObject.properties[i]), true);
    }
    this.updateWindow();
  },

  readObject : function (targetObject)
  {
    var itemResource=this.RDFService.GetResource(targetObject.uri);
    if(this.Container.IndexOf(itemResource)<0) return;

    var property=null;
    for(var i in targetObject.properties){
      property=this.RDFService.GetResource("http://forums.mozillazine.org/rdf/custombutton#"+i);
      var node=this.DataSource.GetTarget(itemResource,property,true);
      targetObject.properties[i] = node.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
    }
  },

  clearData: function()
  {
    var resources=this.Container.GetElements();
    var resource=null;
      while(resources.hasMoreElements()){
      resource=resources.getNext();
      resource=resource.QueryInterface(Components.interfaces.nsIRDFNode);
      this.Container.RemoveElement(resource,false);
      this.Unassert(this.DataSource,resource);
     }
    this.updateWindow();
  },

  move :function(FromURI,ToURI)
  {
    if(! this.RDFService || ! this.DataSource || ! this.Container) this.init();
    var FromResource=this.RDFService.GetResource(FromURI);
    var ToResource=this.RDFService.GetResource(ToURI);
    var FromIndex=this.Container.IndexOf(FromResource);
    var ToIndex=this.Container.IndexOf(ToResource);
    
    if(FromIndex<0 || ToIndex<0) return;

    this.Container.RemoveElement(FromResource,true);
    this.Container.InsertElementAt(FromResource,ToIndex,true);

    this.updateWindow();
  },
  updateWindow : function()
  {
    var browserWindows= null;
    var browserWindow=null;
    var buttonbox=null;

    if( ! this.nsIWindowMediator) 
      this.nsIWindowMediator = Components.classes['@mozilla.org/appshell/window-mediator;1']
                  .getService(Components.interfaces.nsIWindowMediator);

    try{
      //browserWindows=this.nsIWindowMediator.getZOrderDOMWindowEnumerator('navigator:browser',true);
      browserWindows=this.nsIWindowMediator.getZOrderDOMWindowEnumerator('',true);
    }catch(e){ return;}

    while(browserWindows.hasMoreElements())
    {
      browserWindow=browserWindows.getNext();
      buttonbox  =  browserWindow.top.document.getElementById("custombutton-container");
		
      if(buttonbox)
      {
		buttonbox.ref="urn:custombutton:buttons";
        buttonbox.hidden=true;
        buttonbox.builder.rebuild();
        buttonbox.hidden=false;
        buttonbox=null;
      }
     }
  },

  //Independent Utility
  
  Unassert: function (dsource,node)
  {
      var props=dsource.ArcLabelsOut(node);
      while(props.hasMoreElements()){
          var prop=props.getNext();
      var target=dsource.GetTarget(node,prop,true);
         dsource.Unassert(node,prop,target);
      }
  },


  IDmaker : function()
  {
    var d= new Date();
    return d.getTime().toString();
  }
};